
#include "sdk/amxxmodule.h"
#include "HappyHTTP/happyhttp.h"
#include "include/thttp.h"

void THttp_OnBegin(const happyhttp::Response *pResponse, void *pUserData)
{
	THttp *pHandle = (THttp *)pUserData;


	pHandle->m_iHttpCode	= pResponse->getstatus();
	pHandle->m_szHttpReason	= (char *)pResponse->getreason();
	pHandle->m_szResponse	= 0;
	pHandle->m_iResponseSize	= 0;
	pHandle->m_iError	= 0;
//	pHandle->m_szTarget	= 0;

#ifdef GAME_DEBUG
	MF_Log("THttp: Start %x (HTTP %d: %s)", pUserData, pHandle->m_iHttpCode, pHandle->m_szHttpReason);
#endif

}

void THttp_OnData(const happyhttp::Response *pResponse, void *pUserData, const unsigned char *pData, int iSize)
{
	THttp *pHandle = (THttp *)pUserData;

	pHandle->m_iHttpCode = pResponse->getstatus();
	pHandle->m_szHttpReason = (char *)pResponse->getreason();

	if(pHandle->m_iResponseSize + iSize < 3096)
	{
		pHandle->m_szResponse = (char *)realloc(pHandle->m_szResponse, (pHandle->m_iResponseSize + iSize + 1) * sizeof(char));
		memcpy(pHandle->m_szResponse + pHandle->m_iResponseSize, (const char *)pData, iSize);
		pHandle->m_szResponse[pHandle->m_iResponseSize + iSize] = '\0';
		pHandle->m_iResponseSize += iSize;
	}

#ifdef GAME_DEBUG
//	MF_Log("THttp: Received %x %d bytes: %s", pUserData, iSize, (unsigned char *)pHandle->m_szResponse);
#endif
}

void THttp_OnComplete(const happyhttp::Response *pResponse, void *pUserData)
{
	THttp *pHandle = (THttp *)pUserData;

	pHandle->m_iHttpCode = pResponse->getstatus();
	pHandle->m_szHttpReason = (char *)pResponse->getreason();

#ifdef GAME_DEBUG
	MF_Log("THttp: Received %x: %s", pUserData, pHandle->m_szResponse);
	MF_Log("THttp: Finished %x (%d bytes total)", pUserData, pHandle->m_iResponseSize);
#endif

	pHandle->m_iError = 1;
}

void THttp::Request(const char* target, const char* host, const char* method, const char* url, const char* headers[], const unsigned char* body, int bodysize, int timeout)
{
	m_tTimeout = time(NULL) + timeout;
	m_pLink = happyhttp::Connection(host, 80);
	m_pLink.setcallbacks(THttp_OnBegin, THttp_OnData, THttp_OnComplete, this);
//	m_szTarget = new char[(strlen(target) + 1) * sizeof(char *)];
//	strcat(m_szTarget, target);
	snprintf(m_szTarget, 63, "%s", target);
	m_pLink.request(method, url, headers, body, bodysize);
}

void THttp::Request(const char* target, const char* host, const char* method, const char* url, int timeout)
{
	m_tTimeout = time(NULL) + timeout;
	m_pLink = happyhttp::Connection(host, 80);
	m_pLink.setcallbacks(THttp_OnBegin, THttp_OnData, THttp_OnComplete, this);
//	m_szTarget = new char[(strlen(target) + 1) * sizeof(char *)];
//	strcat(m_szTarget, target);
	snprintf(m_szTarget, 63, "%s", target);
	m_pLink.request(method, url);
}

void THttp::CancelHttp()
{
	m_pLink.close();
	m_iResponseSize = 0;
	THttp_OnComplete(NULL, this);
};

bool THttp::Poll()
{
	if(m_pLink.outstanding())
		m_pLink.pump();

	return FALSE;
}

bool THttp::Active()
{
	return (m_iError == 0);
}
